/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import javafx.scene.effect.SepiaTone;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringPool;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.entity.Region;
import org.springblade.system.excel.RegionExcel;
import org.springblade.system.mapper.RegionMapper;
import org.springblade.system.service.IRegionService;
import org.springblade.system.vo.RegionAllVO;
import org.springblade.system.vo.RegionVO;
import org.springframework.stereotype.Service;

import java.util.*;

import static org.springblade.system.cache.RegionCache.*;


/**
 * 行政区划表 服务实现类
 *
 * @author Chill
 */
@Slf4j
@Service
public class RegionServiceImpl extends ServiceImpl<RegionMapper, Region> implements IRegionService {

	@Override
	public boolean submit(Region region) {
		// 设置市级编号格式
		String regionCode = region.getCode();
		String regionParentCode = region.getParentCode();
		if (regionCode.startsWith(MAIN_CODE)) {
			region.setCode(StringUtil.removePrefix(regionCode, MAIN_CODE));
		}
		if (regionParentCode.startsWith(MAIN_CODE)) {
			region.setParentCode(StringUtil.removePrefix(regionParentCode, MAIN_CODE));
		}
		// 查询是否已存在
		Long cnt = baseMapper.selectCount(Wrappers.<Region>query().lambda().eq(Region::getCode, region.getCode()));
		if (cnt > 0L) {
			return this.updateById(region);
		}
		// 设置祖区划编号
		Region parent = getByCode(region.getParentCode());
		if (Func.isNotEmpty(parent) || Func.isNotEmpty(parent.getCode())) {
			String ancestors = parent.getAncestors() + StringPool.COMMA + parent.getCode();
			region.setAncestors(ancestors);
		}
		// 设置省、市、区、镇、村
		Integer level = region.getRegionLevel();
		String code = region.getCode();
		String name = region.getName();
		if (level == PROVINCE_LEVEL) {
			region.setProvinceCode(code);
			region.setProvinceName(name);
		} else if (level == CITY_LEVEL) {
			region.setCityCode(code);
			region.setCityName(name);
		} else if (level == DISTRICT_LEVEL) {
			region.setDistrictCode(code);
			region.setDistrictName(name);
		} else if (level == TOWN_LEVEL) {
			region.setTownCode(code);
			region.setTownName(name);
		} else if (level == VILLAGE_LEVEL) {
			region.setVillageCode(code);
			region.setVillageName(name);
		}
		return this.save(region);
	}

	@Override
	public boolean removeRegion(String id) {
		Long cnt = baseMapper.selectCount(Wrappers.<Region>query().lambda().eq(Region::getParentCode, id));
		if (cnt > 0L) {
			throw new ServiceException("请先删除子节点!");
		}
		return removeById(id);
	}

	@Override
	public List<RegionVO> lazyList(String parentCode, Map<String, Object> param) {
		return baseMapper.lazyList(parentCode, param);
	}


	@Override
	public List<RegionAllVO> lazyListAll() {
		List<RegionAllVO> list = new ArrayList<>();

		Map<String,List<RegionVO>> listMap = new HashMap<>();
		List<RegionVO> regionVOS = baseMapper.lazyListAll();
		//处理省
		for (RegionVO regionVO : regionVOS) {
			if(listMap.size() < 1){
				//没有数据
				List<RegionVO> list1 = new ArrayList<>();
				list1.add(regionVO);
				listMap.put(regionVO.getProvinceCode(),list1);
			}else{
				//有数据
				List<RegionVO> regionVOS2 = listMap.get(regionVO.getProvinceCode());
				if(ObjectUtils.isNotNull(regionVOS2)){
					//有一样的数据
					List<RegionVO> regionVOS1 = listMap.get(regionVO.getProvinceCode());
					regionVOS1.add(regionVO);
					listMap.put(regionVO.getProvinceCode(),regionVOS1);
				}else{
					//没有
					List<RegionVO> list1 = new ArrayList<>();
					list1.add(regionVO);
					listMap.put(regionVO.getProvinceCode(),list1);
				}
			}
		}

//		Map<String, RegionVO> collect = regionVOS.stream().collect(Collectors.toMap(RegionVO::getProvinceCode, Function.identity(), (key1, key2) -> key2));
		Map<String,List<RegionAllVO>> m = new HashMap<>(); //市，区数据
		//处理区
		listMap.forEach((k,v) ->{
			RegionAllVO sheng = new RegionAllVO();

			sheng.setValue(v.get(0).getProvinceCode());
			sheng.setLabel(v.get(0).getProvinceName());

			Set<RegionAllVO> lis = new HashSet<>();
			v.forEach(it ->{
				if(ObjectUtils.isNotNull(it.getCityCode())){
					RegionAllVO shi = new RegionAllVO();
					shi.setValue(it.getCityCode());
					shi.setLabel(it.getCityName());
					lis.add(shi);
					List<RegionAllVO> voList = new ArrayList<>();
					RegionAllVO qu = new RegionAllVO();
					qu.setValue(it.getDistrictCode());
					qu.setLabel(it.getDistrictName());
					voList.add(qu);
					if(m.size() < 1){
						//没有数据 添加市 区数据

						m.put(it.getCityCode(),voList);
					}else{
						//有数据 判断是不是同一个市
						List<RegionAllVO> voList2 = m.get(it.getCityCode());
						if(ObjectUtils.isNotNull(voList2)){
							//有一样的数据
							List<RegionAllVO> voList1 = m.get(it.getCityCode());
							voList1.add(qu);
							m.put(it.getCityCode(),voList1);
						}else{
							//没有
							shi.setValue(it.getCityCode());
							shi.setLabel(it.getCityName());
							m.put(it.getCityCode(),voList);
						}
					}
				}

			});
			//有市对区的数据
			lis.forEach(i ->{
				List<RegionAllVO> voLis = m.get(i.getValue());
				i.setChildren(voLis);
			});
			sheng.setChildren( new ArrayList<>(lis)  );
			list.add(sheng);
		});

		return list;
	}

	@Override
	public List<RegionVO> lazyTree(String parentCode, Map<String, Object> param) {
		return baseMapper.lazyTree(parentCode, param);
	}

	@Override
	public void importRegion(List<RegionExcel> data, Boolean isCovered) {
		List<Region> list = new ArrayList<>();
		data.forEach(regionExcel -> {
			Region region = BeanUtil.copy(regionExcel, Region.class);
			list.add(region);
		});
		if (isCovered) {
			this.saveOrUpdateBatch(list);
		} else {
			this.saveBatch(list);
		}
	}

	@Override
	public List<RegionExcel> exportRegion(Wrapper<Region> queryWrapper) {
		return baseMapper.exportRegion(queryWrapper);
	}
}
